using System.IO;
using Osci.Interfaces;

namespace Osci.Common
{
    /// <summary> Diese Klasse stellt die Standard-Implementierung der abstrakten
    /// OSCIDataSource-Klasse dar. Diese Implementierung puffert Inhaltsdaten
    /// bis zu einer konfigurierbaren Anzahl von Bytes im Arbeitsspeicher.
    /// Wird diese Anzahl überschritten, werden die gepufferten Bytes wie alle
    /// folgenden Bytes in eine temporäre Datei geschrieben.<p></p><p></p>
    /// Zur Dokumentation der Methoden s. OSCIDataSource.
    /// 
    /// <p>Copyright © 2018 Freie Hansestadt Bremen</p>
    /// <p>Erstellt von Governikus GmbH &amp; Co. KG</p>
    /// <p>Diese Bibliothek kann von jedermann nach Maßgabe der Bremer Lizenz für
    /// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen können
    /// unter der URL www.osci.de abgerufen oder bei der
    /// Koordinierungsstelle für IT-Standards (KoSIT), Schillerstr. 1,
    /// 28195 Bremen schriftlich angefordert werden.</p>
    /// 
    /// <p>Author: P. Ricklefs, N. Büngener</p> 
    /// <p>Version: 1.8</p>
    /// </summary>
    /// <seealso cref="OsciDataSource">
    /// </seealso>
    public class SwapBuffer 
        : OsciDataSource
    {

        /// <summary>
        /// <p>Copyright © 2018 Freie Hansestadt Bremen</p>
        /// <p>Erstellt von Governikus GmbH &amp; Co. KG</p>
        /// <p>Diese Bibliothek kann von jedermann nach Maßgabe der Bremer Lizenz für
        /// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen können
        /// unter der URL www.osci.de abgerufen oder bei der
        /// Koordinierungsstelle für IT-Standards (KoSIT), Schillerstr. 1,
        /// 28195 Bremen schriftlich angefordert werden.</p>
        /// 
        /// <p>Author: P. Ricklefs, N. Büngener</p> 
        /// <p>Version: 1.8</p>
        /// </summary>
        public class MStream 
            : MemoryStream
        {
            public override void Close()
            {
                Seek(0, SeekOrigin.Begin);
            }
        }

        private readonly MStream _os;
        private bool _isFirstCall = true;

        public SwapBuffer()
        {
            _os = new MStream();
        }

        public override  Stream OutputStream
        {
            get
            {
                return _os;
            }
        }

        public override  Stream InputStream
        {
            get
            {
                return _os;
            }
        }

        /// <summary> Liefert die Anzahl der (momentan) gespeicherten Bytes.
        /// </summary>
        /// <value> Anzahl der Bytes
        /// </value>
        public override  long Length
        {
            get
            {
                return _os.Length;
            }
        }

        /// <summary> Liefert den Namen des Herstellers.
        /// </summary>
        /// <value> Herstellername
        /// </value>
        public override string Vendor
        {
            get
            {
                return "Governikus";
            }
        }

        /// <summary> Liefert die Versionsnummer
        /// </summary>
        /// <value> Versionsnummer
        /// </value>
        public override  string Version
        {
            get
            {
                return "0.1";
            }
        }

        public override OsciDataSource NewInstance()
        {
            return new SwapBuffer();
        }
        public void Close()
        {
            _os.Close();
        }
    }
}