using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Runtime.CompilerServices;
using Osci.Common;
using Osci.Interfaces;

namespace Osci.Helper
{
    /// <summary> Diese Klasse führt die Kanonisierung gemäß der Spezifikation
    /// http://www.w3.org/TR/xml-c14n durch. Die Funktion beschränkt
    /// sich auf die Anforderungen der OSCI 1.2 Transportbibliothek.
    /// 
    /// <p>Copyright © 2018 Freie Hansestadt Bremen</p>
    /// <p>Erstellt von Governikus GmbH &amp; Co. KG</p>
    /// <p>Diese Bibliothek kann von jedermann nach Maßgabe der Bremer Lizenz für
    /// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen können
    /// unter der URL www.osci.de abgerufen oder bei der
    /// Koordinierungsstelle für IT-Standards (KoSIT), Schillerstr. 1,
    /// 28195 Bremen schriftlich angefordert werden.</p>
    /// 
    /// <p>Author: H. Tabrizi / N. Büngener</p> 
    /// <p>Version: 1.8</p>
    /// </summary>
    /// <seealso cref="DialogFinder">
    /// </seealso>
    public sealed class Canonizer 
        : MemoryStream
    {
        /// <summary>Default parser name. 
        /// </summary>
        //  protected static final String DEFAULT_PARSER_NAME = "org.apache.xerces.parsers.SAXParser";
        private static readonly string _defaultEncoding = Constants.CharEncoding;

        private readonly CanParser _canParser;

        public Hashtable DigestValues
        {
            get
            {
                return _canParser.DigestValues;
            }
        }

        public List<byte[]> SignedInfos
        {
            get
            {
                return _canParser.SignedInfos;
            }
        }

        public List<string> SignedProperties
        {
            get
            {
                return _canParser.SignedProperties;
            }
        }

        public List<string> ContainerNs
        {
            get
            {
                return _canParser.CocoNs;
            }
        }

        public Canonizer(Stream stream, StoreInputStream storeInputStream)
            : this(stream, storeInputStream, true)
        {
        }

        public Canonizer(Stream stream, StoreInputStream storeInputStream, bool isDuplicateIdCheckEnabled)
        {
            _canParser = new CanParser(this, storeInputStream)
            {
                IsDuplicateIdCheckEnabled =  isDuplicateIdCheckEnabled
            };
            _canParser.StartCanonicalization(stream, false);

            Position = 0;
        }
    }
}