using System.IO;
using Osci.Common;
using Osci.Extensions;

namespace Osci.Helper
{
    /// <summary>
    /// <p>Copyright © 2018 Freie Hansestadt Bremen</p>
    /// <p>Erstellt von Governikus GmbH &amp; Co. KG</p>
    /// <p>Diese Bibliothek kann von jedermann nach Maßgabe der Bremer Lizenz für
    /// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen können
    /// unter der URL www.osci.de abgerufen oder bei der
    /// Koordinierungsstelle für IT-Standards (KoSIT), Schillerstr. 1,
    /// 28195 Bremen schriftlich angefordert werden.</p>
    /// 
    /// <p>Author: N. Büngener</p> 
    /// <p>Version: 1.8</p>
    /// </summary>
    public class Tools
    {
        public static bool CompareByteArrays(byte[] a, byte[] b)
        {
            if (a.Length != b.Length)
                return false;

            for (int i = 0; i < a.Length; i++)
            {
                if (a[i] != b[i])
                    return false;
            }

            return true;
        }

        public static X509Certificate CreateCertificateFromBase64String(string base64String)
        {
            using (MemoryStream memoryStream = new MemoryStream(base64String.ToByteArray()))
            {
                using (Base64InputStream base64InputStream = new Base64InputStream(memoryStream))
                {
                    return new X509Certificate(base64InputStream);
                }
            }
        }

        /// <summary> Erzeugen eines Zertifikates.
        /// </summary>
        /// <param name="path">Path des einzulesenden Zertifikates
        /// </param>
        /// <returns> X509Certificate
        /// @throws CryptographicException
        /// </returns>
        public static X509Certificate CreateCertificate(string path)
        {
            byte[] data = File.ReadAllBytes(path);
            return new X509Certificate(data);
        }


        public static string CreateRandom(int length)
        {
            SupportClass.SecureRandomSupport random = new SupportClass.SecureRandomSupport();
            byte[] bytes = new byte[length];
            bytes = random.NextBytes(bytes);
            return Base64.Encode(bytes);
        }
    }
}