using Osci.Helper;

namespace Osci.MessageParts
{
	/// <summary><H4>FeedbackObject</H4>
	/// <p>Diese Klasse repräsentiert die Feedback-Einträge einer OSCIResponseTo-Nachricht</p>
	/// <p>Copyright © 2018 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Maßgabe der Bremer Lizenz für
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen können
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle für IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Büngener</p> 
	/// <p>Version: 1.8</p>
	/// </summary>
	public class FeedbackObject
	{
		private static Log _log = LogFactory.GetLog(typeof(FeedbackObject));
		private string _lang, _code, _text;

		internal FeedbackObject(string[] feedback) {
			_lang = feedback[0];
			_code = feedback[1];
			_text = feedback[2];
		}

		/// <summary> Liefert die Sprachkürzel der Feedbackeinträge (de, en...).
		/// </summary>
		/// <value> Sprachkürzel
		/// </value>
		public string Language {get{return _lang;}}

		/// <summary> Liefert die Rückmeldecodes der Feedbackeinträge.
		/// </summary>
		/// <value> Rückmeldecodes
		/// </value>
		public string Code {get{return _code;}}

		/// <summary> Liefert die Texte der Feedbackeinträge.
		/// </summary>
		/// <value> Texte
		/// </value>
		public string Text {get{return _text;}}
	}
}
