using Osci.MessageParts;

namespace Osci.Messagetypes
{
    /// <summary> Dieses Interface wird von allen Nachrichtentypen implementiert, die
    /// Inhaltsdaten enthalten.
    /// 
    /// <p>Copyright © 2018 Freie Hansestadt Bremen</p>
    /// <p>Erstellt von Governikus GmbH &amp; Co. KG</p>
    /// <p>Diese Bibliothek kann von jedermann nach Maßgabe der Bremer Lizenz für
    /// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen können
    /// unter der URL www.osci.de abgerufen oder bei der
    /// Koordinierungsstelle für IT-Standards (KoSIT), Schillerstr. 1,
    /// 28195 Bremen schriftlich angefordert werden.</p>
    /// 
    /// <p>Author: P. Ricklefs, N. Büngener</p> 
    /// <p>Version: 1.8</p>
    /// </summary>
    public interface IContentPackage
    {
        /// <summary>
        /// Liefert die Message-ID der Nachricht (Antwort). 
        /// </summary>
        /// <value>Message-ID 
        /// </value>
        string MessageId
        {
            get;
        }

        /// <summary>
        /// Liefert den im Antwortlaufzettel enthaltenen Betreff-Eintrag. 
        /// </summary>
        /// <value>Betreff der Antwortnachricht
        /// </value>
        string Subject
        {
            get;
        }

        /// <summary>
        /// Liefert die in die Nachricht eingestellten (unverschlüsselten) Inhaltsdaten als ContentContainer-Objekte.
        /// </summary>
        /// <value>enthaltene ContentContainer mit Inhaltsdaten
        /// </value>
        ContentContainer[] ContentContainer
        {
            get;
        }

        /// <summary>
        /// Liefert die in die Nachricht eingestellten verschlüsselten Inhaltsdaten 
        /// </summary>
        /// <value>enthaltene EncryptedData-Objekt mit verschlüsselten Inhaltsdaten
        /// </value>
        EncryptedDataOsci[] EncryptedData
        {
            get;
        }
    }
}