using Osci.Common;
using Osci.MessageParts;

namespace Osci.Messagetypes
{
    /// <summary>Diese Klasse dient dazu, von außerhalb des Packages auf Konstruktoren und
    /// Methoden zuzugreifen, die dort nicht sichtbar sind, um die API für den Anwender
    /// übersichtlich zu halten. Die Klasse sollte deshalb nicht dokumentiert werden.
    /// 
    /// <p>Copyright © 2018 Freie Hansestadt Bremen</p>
    /// <p>Erstellt von Governikus GmbH &amp; Co. KG</p>
    /// <p>Diese Bibliothek kann von jedermann nach Maßgabe der Bremer Lizenz für
    /// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen können
    /// unter der URL www.osci.de abgerufen oder bei der
    /// Koordinierungsstelle für IT-Standards (KoSIT), Schillerstr. 1,
    /// 28195 Bremen schriftlich angefordert werden.</p>
    /// 
    /// <p>Author: P. Ricklefs, N. Büngener</p> 
    /// <p>Version: 1.8</p>
    /// </summary>
    /// <seealso cref="ContentContainer"></seealso>
    /// <seealso cref="Attachment"></seealso>
    public class MessageTypesEntry
    {

        public MessageTypesEntry()
        {
            ;
        }

        public static ResponseToInitDialog ResponseToInitDialog(InitDialog id)
        {
            return new ResponseToInitDialog(id);
        }

        public static ResponseToExitDialog ResponseToExitDialog(DialogHandler dH)
        {
            return new ResponseToExitDialog(dH);
        }

        public static ResponseToGetMessageId ResponseToGetMessageId(DialogHandler dH)
        {
            return new ResponseToGetMessageId(dH);
        }

        public static ResponseToStoreDelivery ResponseToStoreDelivery(DialogHandler dH)
        {
            return new ResponseToStoreDelivery(dH, false);
        }

        public static ResponseToFetchDelivery ResponseToFetchDelivery(FetchDelivery fd, StoreDelivery sd)
        {
            return new ResponseToFetchDelivery(fd, sd);
        }

        public static ResponseToFetchProcessCard ResponseToFetchProcessCard(FetchProcessCard fpc)
        {
            return new ResponseToFetchProcessCard(fpc);
        }

        public static ResponseToMediateDelivery ResponseToMediateDelivery(MediateDelivery md, ResponseToProcessDelivery rpd)
        {
            return new ResponseToMediateDelivery(md, rpd);
        }

        public static ResponseToForwardDelivery ResponseToForwardDelivery(AcceptDelivery ad, ResponseToAcceptDelivery rad)
        {
            return new ResponseToForwardDelivery(ad, rad);
        }

        public static ProcessDelivery ProcessDelivery(MediateDelivery md)
        {
            return new ProcessDelivery(md);
        }

        public static AcceptDelivery AcceptDelivery(ForwardDelivery fd)
        {
            return new AcceptDelivery(fd);
        }

        public static void SetFeedBack(OsciResponseTo rsp, string[] codes)
        {
            rsp.SetFeedback(codes);
        }

        public static void Sign(OsciMessage msg)
        {
            if (msg is OsciRequest)
            {
                ((OsciRequest) msg).Sign();
            }
            else
            {
                ((OsciResponseTo)msg).Sign();
            }
        }

        public static void WriteXml(OsciMessage rsp, System.IO.Stream out0)
        {
            rsp.WriteXml(out0);
        }
        public static void AddContentContainer(OsciMessage msg, ContentContainer coco)
        {
            msg.AddContentContainer(coco);
        }
    }
}
