using System;
using Osci.Cryptographic;
using Osci.Helper;

namespace Osci.Roles
{
    /// <summary> Diese Klasse stellt einen OSCI-Intermediär dar.
    /// 
    /// <p>Copyright © 2018 Freie Hansestadt Bremen</p>
    /// <p>Erstellt von Governikus GmbH &amp; Co. KG</p>
    /// <p>Diese Bibliothek kann von jedermann nach Maßgabe der Bremer Lizenz für
    /// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen können
    /// unter der URL www.osci.de abgerufen oder bei der
    /// Koordinierungsstelle für IT-Standards (KoSIT), Schillerstr. 1,
    /// 28195 Bremen schriftlich angefordert werden.</p>
    /// 
    /// <p>Author: P. Ricklefs, N. Büngener</p> 
    /// <p>Version: 1.8</p>
    /// </summary>
    public class Intermed
        : Role
    {
        /// <summary> Ruft die URI des Intermediärs ab, oder legt diese fest.
        /// </summary> 
        /// <value> URI
        /// </value>
        public Uri Uri
        {
            get; set;
        }

        /// <summary> Konstruktor für ein Intermediärs-Objekt (intermediärsseitig).
        /// </summary>
        /// <param name="signer">Signer-Objekt, welches die Signaturen der Nachrichten
        /// erstellen soll (null, wenn keine Signaturen gewünscht).
        /// </param>
        /// <param name="decrypter">Decrypter-Objekt, welches den Inhalt der Nachrichten
        /// entschlüsseln soll.
        /// </param>
        public Intermed(Signer signer, Decrypter decrypter)
        {
            Signer = signer;
            Decrypter = decrypter;
        }

        /// <summary> Konstruktor für ein Intermediärs-Objekt (clientseitig).
        /// Das Signaturzertifikat des Intermediärs muss nicht übergeben werden.
        /// Die verwendete URL muss durch die Transportimplementierung aufgelöst werden können.
        /// </summary>
        /// <param name="signatureCertificate">Zertifikat, mit dem die Signatur einer Antwort
        /// geprüft werden kann.
        /// </param>
        /// <param name="cipherCertificate">Zertifikat, mit dem die Nachricht verschlüsselt
        /// werden soll.
        /// </param>
        /// <param name="uri">URL des Intermediärs.
        /// </param>
        public Intermed(X509Certificate signatureCertificate, X509Certificate cipherCertificate, Uri uri)
        {
            SignatureCertificate = signatureCertificate;
            CipherCertificate = cipherCertificate;
            Uri = uri;
        }

    }
}